export declare const DIGIT_CHARSET = "0123456789";
export declare const LOWERCASE_CHARSET = "abcdefghijklmnopqrstuvwxyz";
export declare const UPPERCASE_CHARSET = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
export declare const SYMBOL_CHARSET = "!\"#$%&'()*+,-./:;<=>?@[]{}^_`|~";
export type PasswordOptionType = boolean | number | {
    min: number;
};
export type PasswordOptionsType = {
    length: number;
    digits: PasswordOptionType;
    symbols: PasswordOptionType;
    lowercase: PasswordOptionType;
    uppercase: PasswordOptionType;
};
/**
 * Generate a random password.
 *
 * Uses a CSPRNG for randomness.
 *
 *     generatePassword(); // l[Nz8UfU.o4g
 *     generatePassword({ length: 8 }); // i&n4Htp=
 *     generatePassword({ length: 8, symbols: false, digits: 2 }); // k9WTkaP6
 *     generatePassword({ length: 8, digits: {min: 2} }); // 0(c69+.f
 *
 * @param options
 * @param options.length The length of the resulting password. Defaults to 12.
 * @param options.digits Include or exclude digits.
 * @param options.symbols Include or exclude symbols.
 * @param options.lowercase Include or exclude lowercase.
 * @param options.uppercase Include or exclude uppercase.
 * @returns A random password.
 */
export declare function generatePassword(options?: Partial<PasswordOptionsType>): string;
/**
 * Generate a random digit pin.
 *
 * Uses a CSPRNG for randomness.
 *
 *     generatePin(6); // 036919
 *     generatePin(8); // 45958396
 *
 * @param length The length of the resulting pin.
 * @returns A random digit pin.
 */
export declare function generatePin(length: number): string;
/**
 * Generate a string of `length` characters chosen randomly from the given `charset`.
 *
 * Uses a CSPRNG for randomness.
 *
 *     generateCharacters(4, '$%^&');                          // &$&^
 *     generateCharacters(6, '0123456789');                    // 947682
 *     generateCharacters(6, 'abcdefghijklmnopqrstuvwxyz');    // ihdrnn
 *
 * @param length The number of random characters to generate.
 * @param charset The set of characters to randomly sample from.
 * @returns A random string of `length` characters from `charset`.
 */
export declare function generateCharacters(length: number, charset: string): string;
/**
 * Randomize the ordering of the characters in the given string.
 *
 * Uses a CSPRNG for randomness.
 *
 *     randomizeCharacters('randomize me');     // e znmaedimro
 *     randomizeCharacters('randomize me');     // arndimz moee
 *     randomizeCharacters('randomize me');     // ai emdonmrze
 *
 * @param characters A string of characters to randomize.
 * @returns A random ordering of the `characters` argument.
 */
export declare function randomizeCharacters(characters: string): string;
